package uprrp.tania;

import android.app.Application;
import android.content.Intent;
import android.content.SharedPreferences;
import android.util.Log;
import android.widget.Toast;

import androidx.annotation.NonNull;

import com.amazonaws.auth.CognitoCachingCredentialsProvider;
import com.amazonaws.regions.Regions;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.messaging.FirebaseMessaging;

import org.researchstack.backbone.StorageAccess;
import org.researchstack.backbone.storage.database.AppDatabase;
import org.researchstack.backbone.storage.database.sqlite.DatabaseHelper;
import org.researchstack.backbone.storage.file.EncryptionProvider;
import org.researchstack.backbone.storage.file.FileAccess;
import org.researchstack.backbone.storage.file.PinCodeConfig;
import org.researchstack.backbone.storage.file.SimpleFileAccess;
import org.researchstack.backbone.storage.file.UnencryptedProvider;

import uprrp.tania.activities.GettingStartedActivity;

public class App extends Application {

    private static final String TAG = "App";
    private static final String IDENTITY_POOL_ID = "us-east-1:574094cd-0784-4e26-bd14-4fa72ae63579";
    private static final String PREFS_NAME = "prefs";
    private static final String NEEDS_TO_REGISTER_KEY = "needsToRegister";

    @Override
    public void onCreate() {
        super.onCreate();

        initializeStorageAccess();
        retrieveFirebaseToken();
        checkAndRedirectForRegistration();
    }

    private void initializeStorageAccess() {
        PinCodeConfig pinCodeConfig = new PinCodeConfig();
        EncryptionProvider encryptionProvider = new UnencryptedProvider();
        FileAccess fileAccess = new SimpleFileAccess();
        AppDatabase database = new DatabaseHelper(this,
                DatabaseHelper.DEFAULT_NAME,
                null,
                DatabaseHelper.DEFAULT_VERSION);

        StorageAccess.getInstance().init(pinCodeConfig, encryptionProvider, fileAccess, database);
    }

    private void retrieveFirebaseToken() {
        Toast.makeText(getApplicationContext(), "Waiting for token...", Toast.LENGTH_LONG).show();

        FirebaseMessaging.getInstance().getToken()
                .addOnCompleteListener(new OnCompleteListener<String>() {
                    @Override
                    public void onComplete(@NonNull Task<String> task) {
                        if (!task.isSuccessful()) {
                            Log.w(TAG, "Fetching FCM registration token failed", task.getException());
                            Toast.makeText(getApplicationContext(), "Couldn't retrieve token!", Toast.LENGTH_LONG).show();
                            return;
                        }

                        String token = task.getResult();
                        GlobalValues.getInstance().setDeviceToken(token);
                        Log.d(TAG, "Got token: " + token);

                        Toast.makeText(getApplicationContext(), "Got token!", Toast.LENGTH_LONG).show();

                        registerWithSNS();
                    }
                });
    }

    private void registerWithSNS() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                CognitoCachingCredentialsProvider credentialsProvider = new CognitoCachingCredentialsProvider(
                        getApplicationContext(),
                        IDENTITY_POOL_ID,
                        Regions.US_EAST_1
                );
                SNSRegister snsRegister = new SNSRegister(credentialsProvider);
                snsRegister.execute();
            }
        }).start();
    }

    private void checkAndRedirectForRegistration() {
        SharedPreferences prefs = getSharedPreferences(PREFS_NAME, MODE_PRIVATE);
        boolean needsToRegister = prefs.getBoolean(NEEDS_TO_REGISTER_KEY, true);
        if (needsToRegister) {
            Log.d(TAG, "Redirecting to GettingStarted");
            Intent intent = new Intent(this, GettingStartedActivity.class);
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            startActivity(intent);
        }
    }
}