package uprrp.tania.activities;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Environment;
import android.util.Base64;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import uprrp.tania.GlobalValues;
import uprrp.tania.R;
import uprrp.tania.networking.SendLoginCredentials;
import uprrp.tania.utils.URLEventListener;

// For basic Android functionality
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

// For file operations
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.BufferedInputStream;
import java.io.FileInputStream;

// For Base64 encoding/decoding
import android.util.Base64;

// For PDF creation (using iText)
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.text.Font;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Element;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Image;

// For PDF viewing (using AndroidPdfViewer)
import com.github.barteksc.pdfviewer.PDFView;
import com.github.barteksc.pdfviewer.listener.OnDrawListener;
import com.github.barteksc.pdfviewer.listener.OnLoadCompleteListener;
import com.github.barteksc.pdfviewer.listener.OnPageChangeListener;
import com.github.barteksc.pdfviewer.listener.OnPageErrorListener;
import com.github.barteksc.pdfviewer.listener.OnTapListener;
import com.github.barteksc.pdfviewer.scroll.DefaultScrollHandle;
import com.github.barteksc.pdfviewer.util.FitPolicy;

// For Fragment
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

// For drawing and touch events
import android.graphics.Canvas;
import android.view.MotionEvent;

// Your app's R class for resources
//import your.package.name.R;
public class AccountRecoveryActivity extends AppCompatActivity {

    private static final String TAG = "AccountRecoveryActivity";

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        // Start constructor
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_account_recovery);

        // Change all caps text to normal capitalization and add onClick listeners
        final Button recoverButton = findViewById(R.id.recoverButton);
        final Button forgotPasswordButton = findViewById(R.id.forgotPasswordButton);
        recoverButton.setTransformationMethod(null); // TODO: this is a workaround I found, any other acceptable solution is welcome
        forgotPasswordButton.setTransformationMethod(null); // TODO: this is a workaround I found, any other acceptable solution is welcome
        recoverButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                sendAccountRecoveryRequest();
            }
        });
        forgotPasswordButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent = new Intent(AccountRecoveryActivity.this, ForgotPasswordActivity.class);
                startActivity(intent);
            }
        });

    }

    private void sendAccountRecoveryRequest() {

        final EditText emailText = findViewById(R.id.emailTextRecover);
        final EditText passwordText = findViewById(R.id.passwordTextRecover);

        // Email and password are needed (make sure user knows); else, proceed
        if(emailText.getText().toString().equals("") || passwordText.getText().toString().equals("")) {
            Toast.makeText(AccountRecoveryActivity.this, "All fields are needed!", Toast.LENGTH_LONG).show();
        } else {

            // Initiate progress dialog
            // TODO: find substitute for this deprecated dialog box
            final ProgressDialog progressDialog = ProgressDialog.show(AccountRecoveryActivity.this,
                    "Recovering Account",
                    getString(R.string.progressDialogDescriptionText));

            // Send request to server
            SendLoginCredentials accountRecoveryTask = new SendLoginCredentials(new URLEventListener() {

                @Override
                public void onSuccess(String response) {

                    progressDialog.dismiss();
                    restoreConsentForm(response);

                    // Change user preferences to reflect login
                    Context context = AccountRecoveryActivity.this;
                    SharedPreferences prefs = context.getSharedPreferences("prefs", Context.MODE_PRIVATE);
                    SharedPreferences.Editor editor = prefs.edit();
                    editor.putBoolean("needsToRegister", false);
                    editor.apply();

                    // Start MainActivity
                    Intent intent = new Intent(context, MainActivity.class);
                    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                    context.startActivity(intent);
                    Toast.makeText(getApplicationContext(), "Login successful!", Toast.LENGTH_LONG).show();

                }

                @Override
                public void onFailure(Exception e) {
                    progressDialog.dismiss();
                    Toast.makeText(AccountRecoveryActivity.this, "Couldn't recover account!\nTry again later with the correct credentials...", Toast.LENGTH_LONG).show();
                    Log.e(TAG, "Error occurred while sending account recovery request to server...");
                    e.printStackTrace();
                }

            });

            // Start task
            String deviceToken = GlobalValues.getInstance().getDeviceToken();
            String email = emailText.getText().toString();
            String password = passwordText.getText().toString();
            accountRecoveryTask.execute(deviceToken, email, password);

        }

    }

    // TODO: IO should be moved to a background task
    private void restoreConsentForm(String response) {
        FileOutputStream consentForm = null;
        try {
            byte[] consentBytes = Base64.decode(response, Base64.NO_WRAP);
            File file = new File(getFilesDir(), getString(R.string.consentFileName));
            consentForm = new FileOutputStream(file);
            consentForm.write(consentBytes);
            Log.d(TAG, "Consent form restored successfully");
        } catch (IllegalArgumentException e) {
            Log.e(TAG, "Invalid Base64 string", e);
        } catch (IOException e) {
            Log.e(TAG, "Error writing consent form", e);
        } finally {
            if (consentForm != null) {
                try {
                    consentForm.close();
                } catch (IOException e) {
                    Log.e(TAG, "Error closing FileOutputStream", e);
                }
            }
        }
    }
}
