package uprrp.tania.networking;

import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

import javax.net.ssl.HttpsURLConnection;

import uprrp.tania.utils.URLEventListener;

public class SendConsentForm {

    private static final String TAG = "SendConsentForm";
    private static final String consentFormBaseURL = "https://tania.uprrp.edu/getSignatureIOS.php";
    private final URLEventListener myCallback;
    private final Executor executor = Executors.newSingleThreadExecutor();

    public SendConsentForm(URLEventListener callback) {
        this.myCallback = callback;
    }

    private boolean validInputs(String... strings) {
        if (strings == null || strings.length != 3) {
            Log.e(TAG, "Invalid string array length!");
            return false;
        }

        for (int i = 0; i < strings.length; i++) {
            if (strings[i] == null || strings[i].isEmpty()) {
                Log.e(TAG, "Encountered null or empty parameter on index " + i);
                return false;
            }
        }

        return true;
    }
    public void execute(String... strings) {
        executor.execute(() -> {
            String result = doInBackground(strings);
            onPostExecute(result);
        });
    }

//    private String doInBackground(String... strings) {
//        // Validation
//        if(!validInputs(strings)) {
//            Log.e(TAG, "Invalid inputs given!");
//            return null;
//        }
//
//        // Extract variables
//        String deviceToken = strings[0];
//        String consentFormBase64 = strings[1];
//        String signatureDate = strings[2];
//
//        try {
//            // Create JSON
//            JSONObject consentFormJSON = new JSONObject();
//            consentFormJSON.put("token", deviceToken);
//            consentFormJSON.put("signatureDate", signatureDate);
//            consentFormJSON.put("signaturePDFBase64", consentFormBase64);
//            Log.d(TAG, "ConsentFormJSON is " + consentFormJSON.toString()); // log
//
//            // Encode data
//            String encodedRequestBody = URLEncoder.encode("data", "UTF-8") + "=" + URLEncoder.encode(consentFormJSON.toString(), "UTF-8");
//
//            // Send POST data request
//            URL url = new URL(consentFormBaseURL);
//            HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();
//            conn.setRequestMethod("POST");
//            conn.setDoOutput(true);
//            conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
//
//            try (Writer writer = new BufferedWriter(new OutputStreamWriter(conn.getOutputStream(), StandardCharsets.UTF_8))) {
//                writer.write(encodedRequestBody);
//            }
//
//            // Get the server response
//            StringBuilder serverResponse = new StringBuilder();
//            try (BufferedReader serverReader = new BufferedReader(new InputStreamReader(conn.getInputStream()))) {
//                String line;
//                while ((line = serverReader.readLine()) != null) {
//                    serverResponse.append(line);
//                }
//            }
//
//            String response = serverResponse.toString();
//            Log.d(TAG, "The server's response is " + response);
//            if(response.toLowerCase().contains("error")) {
//                return null;
//            } else {
//                return response;
//            }
//
//        } catch (JSONException e) {
//            Log.e(TAG, "Couldn't prepare consentFormJSON!", e);
//            return null;
//        } catch (UnsupportedEncodingException e) {
//            Log.e(TAG, "Couldn't encode consentFormJSON!", e);
//            return null;
//        } catch(Exception e) {
//            Log.e(TAG, "Couldn't communicate with server while recovering account!", e);
//            return null;
//        }
//    }
private String doInBackground(String... strings) {
    Log.d(TAG, "doInBackground: Starting method execution");

    // Validation
    if(!validInputs(strings)) {
        Log.e(TAG, "doInBackground: Invalid inputs given!");
        return null;
    }

    // Extract variables
    String deviceToken = strings[0];
    String consentFormBase64 = strings[1];
    String signatureDate = strings[2];

    Log.d(TAG, "doInBackground: Extracted variables - deviceToken length: " + deviceToken.length() +
            ", consentFormBase64 length: " + consentFormBase64.length() +
            ", signatureDate: " + signatureDate);

    HttpsURLConnection conn = null;
    try {
        // Create JSON
        JSONObject consentFormJSON = new JSONObject();
        consentFormJSON.put("token", deviceToken);
        consentFormJSON.put("signatureDate", signatureDate);
        consentFormJSON.put("signaturePDFBase64", consentFormBase64);
        Log.d(TAG, "doInBackground: ConsentFormJSON created successfully");

        // Encode data
        String encodedRequestBody = URLEncoder.encode("data", "UTF-8") + "=" + URLEncoder.encode(consentFormJSON.toString(), "UTF-8");
        Log.d(TAG, "doInBackground: Request body encoded, length: " + encodedRequestBody.length());

        // Send POST data request
        URL url = new URL(consentFormBaseURL);
        Log.d(TAG, "doInBackground: Attempting to connect to URL: " + consentFormBaseURL);

        conn = (HttpsURLConnection) url.openConnection();
        conn.setRequestMethod("POST");
        conn.setDoOutput(true);
        conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
        conn.setConnectTimeout(15000); // 15 seconds timeout
        conn.setReadTimeout(15000); // 15 seconds timeout

        Log.d(TAG, "doInBackground: Connection opened, writing request body");
        try (Writer writer = new BufferedWriter(new OutputStreamWriter(conn.getOutputStream(), StandardCharsets.UTF_8))) {
            writer.write(encodedRequestBody);
        }
        Log.d(TAG, "doInBackground: Request body written successfully");

        // Get the server response
        int responseCode = conn.getResponseCode();
        Log.d(TAG, "doInBackground: Received response code: " + responseCode);

        StringBuilder serverResponse = new StringBuilder();
        try (BufferedReader serverReader = new BufferedReader(new InputStreamReader(
                responseCode >= 400 ? conn.getErrorStream() : conn.getInputStream()))) {
            String line;
            while ((line = serverReader.readLine()) != null) {
                serverResponse.append(line);
            }
        }

        String response = serverResponse.toString();
        Log.d(TAG, "doInBackground: Server response: " + response);

        if(responseCode >= 400 || response.toLowerCase().contains("error")) {
            Log.e(TAG, "doInBackground: Error response received");
            return null;
        } else {
            Log.d(TAG, "doInBackground: Successful response received");
            return response;
        }

    } catch (JSONException e) {
        Log.e(TAG, "doInBackground: Couldn't prepare consentFormJSON", e);
        return null;
    } catch (UnsupportedEncodingException e) {
        Log.e(TAG, "doInBackground: Couldn't encode consentFormJSON", e);
        return null;
    } catch(Exception e) {
        Log.e(TAG, "doInBackground: Couldn't communicate with server", e);
        return null;
    } finally {
        if (conn != null) {
            conn.disconnect();
            Log.d(TAG, "doInBackground: Connection closed");
        }
    }
}
    private void onPostExecute(String response) {
        if(this.myCallback == null) {
            Log.e(TAG, "Callback wasn't initialized first!");
            return;
        }

        if(response == null) {
            this.myCallback.onFailure(new Exception("Error occurred during transaction!"));
        } else {
            this.myCallback.onSuccess();
        }
    }
}